unit StgView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, Storages, ComCtrls, ExtCtrls, ToolWin, Menus, ImgList;

type
  TfmStgView = class(TForm)
    OpenDialog: TOpenDialog;
    ImageList: TImageList;
    tbMain: TToolBar;
    sbConvert: TSpeedButton;
    btnOpen: TSpeedButton;
    tbSeparator1: TToolButton;
    btnSave: TSpeedButton;
    btnFont: TSpeedButton;
    FontDialog: TFontDialog;
    pmTree: TPopupMenu;
    miInsert: TMenuItem;
    miDelete: TMenuItem;
    miInsertStorage: TMenuItem;
    tvStorages: TTreeView;
    mmStreamData: TMemo;
    Splitter: TSplitter;
    tbSeparator2: TToolButton;
    procedure btnOpenClick(Sender: TObject);
    procedure sbConvertClick(Sender: TObject);
    procedure tvStoragesChange(Sender: TObject; Node: TTreeNode);
    procedure btnSaveClick(Sender: TObject);
    procedure btnFontClick(Sender: TObject);
    procedure miInsertClick(Sender: TObject);
    procedure miInsertStorageClick(Sender: TObject);
    procedure tvStoragesEdited(Sender: TObject; Node: TTreeNode;var S: String);
    procedure mmStreamDataExit(Sender: TObject);
  protected
    FCurStream : String;
    procedure AddChildNodes( ALastNode : TTreeNode ; AParentStg : TStorage );
    function GetNodePath( ANode : TTreeNode ) : String;
		function GetStoragePath( ANode : TTreeNode ) : String;
    function GetNodeFullName( ANode : TTreeNode ) : String;
  end;

var
  fmStgView: TfmStgView;

//=============================================================
implementation

uses ActiveX;
{$R *.DFM}

//-------------------------------------------------------------
// GetNodeFullName return full path to storage item
// Example:   'd:\path\file.ext:storage\item'
function TfmStgView.GetNodePath( ANode : TTreeNode ) : String;
begin
	if ANode.Level=0 then Result := ANode.Text+':'
	else
		begin
			Result := '';
			while ANode.Level>0 do
			begin
				Result := ANode.Text+'\'+Result;
				ANode := ANode.Parent;
			end;
			Result := ANode.Text+':'+Result;
		end;
end;
//-------------------------------------------------------------
function TfmStgView.GetStoragePath( ANode : TTreeNode ) : String;
begin
	if ANode.Level=0 then Result := ''
	else
		begin
			Result := '';
			ANode := ANode.Parent;
			while ANode.Level>0 do
			begin
				Result := ANode.Text+'\'+Result;
				ANode := ANode.Parent;
			end;
			Result := ANode.Text+':'+Result;
		end;
end;
//-------------------------------------------------------------
// GetNodeFullName return full path to storage contained this item.
// Example:   'd:\path\file.ext:storage'
function TfmStgView.GetNodeFullName( ANode : TTreeNode ) : String;
begin
	if ANode.Level=0 then Result := ANode.Text+':'
	else
		begin
			Result := ANode.Text;
			ANode := ANode.Parent;
			while ANode.Level>0 do
			begin
				Result := ANode.Text+'\'+Result;
				ANode := ANode.Parent;
			end;
			Result := ANode.Text+':'+Result;
		end;
end;
//-------------------------------------------------------------
procedure TfmStgView.AddChildNodes( ALastNode : TTreeNode ; AParentStg : TStorage );
var
  i : Integer;
  oStg : TStorage;
  oItemNames : TStrings;
  oCurrentNode : TTreeNode;
begin
	if AParentStg=nil then Exit;
	oItemNames := TStringList.Create;
	try
		AParentStg.EnumElements(oItemNames);
		for i := 0 to oItemNames.Count-1 do
		begin
			oCurrentNode := tvStorages.Items.AddChild(ALastNode,oItemNames[i]);
			oCurrentNode.ImageIndex := Integer(oItemNames.Objects[i]);
			oCurrentNode.SelectedIndex := oCurrentNode.ImageIndex;
			if LongInt(oItemNames.Objects[i])=STGTY_STORAGE then
			begin
				oStg := AParentStg.OpenStorage( oItemNames[i], {stgmOpenRead}STGM_READ or STGM_SHARE_EXCLUSIVE );
				if oStg<>nil then AddChildNodes( oCurrentNode, oStg );
			end;
		end;
	finally
		oItemNames.Free;
		AParentStg.Close;
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.btnOpenClick(Sender: TObject);
var
  oRootNode : TTreeNode;
	sFileName : String;
begin
	OpenDialog.Filter := 'All files (*.*)|*.*|MS Word document (*.doc)|*.*|MS Excel workbook (*.*)|*.xlw';
	if OpenDialog.Execute then
	try
		sFileName := OpenDialog.FileName;
		oRootNode := tvStorages.Items.Add(nil,sFileName);
		oRootNode.ImageIndex := 0;
		AddChildNodes( oRootNode, TStgFile.OpenFile( sFileName, {stgmOpenRead}STGM_READ or STGM_SHARE_EXCLUSIVE ) );
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle,	PChar('Error on open storage file '''+sFileName+''''+#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;
//-------------------------------------------------------------
// Convert plain file to strutured storage
procedure TfmStgView.sbConvertClick(Sender: TObject);
var
  oRootNode : TTreeNode;
	sFileName : String;
begin
	OpenDialog.Filter := 'Text files (*.txt)|*.txt|All files (*.*)|*.*';
	if OpenDialog.Execute then
	try
		sFileName := OpenDialog.FileName;
		oRootNode := tvStorages.Items.Add(nil,sFileName);
		oRootNode.ImageIndex := 0;
		AddChildNodes( oRootNode, TStgFile.CreateFile(sFileName,stgmConvert) );
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle,	PChar('Error on convert file '''+sFileName+''' to storage'+#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.tvStoragesChange(Sender: TObject; Node: TTreeNode);
var
  oStream : TStream;
  oCurrentNode : TTreeNode;
begin
	FCurStream := '';
	oCurrentNode := Node;
	if oCurrentNode.ImageIndex<>2 then mmStreamData.Lines.Clear
	else
		try
			oStream := OpenStream( GetNodeFullName(Node), fmOpenReadWrite or fmShareExclusive );
			if oStream<>nil then
			try
				mmStreamData.Lines.LoadFromStream( oStream );
				FCurStream := GetNodeFullName( Node );
			finally
				oStream.Destroy;
			end;
		except
			on E:Exception do
				Windows.MessageBox( Application.Handle, PChar('Error on reading from stream.'#13#10+E.ClassName+': '+E.Message),
					'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
		end;
end;
//-------------------------------------------------------------
procedure TfmStgView.btnSaveClick(Sender: TObject);
var
  oStream : TStream;
begin
	try
		oStream := OpenStream( FCurStream, fmOpenReadWrite or fmShareExclusive );
		if oStream<>nil then
		try
			mmStreamData.Lines.SaveToStream( oStream );
		finally
			oStream.Destroy;
		end;
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle, PChar('Error on writing to stream.'#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.btnFontClick(Sender: TObject);
begin
	if FontDialog.Execute then mmStreamData.Font := FontDialog.Font;
end;
//-------------------------------------------------------------
procedure TfmStgView.miInsertClick(Sender: TObject);
var
  sName : String;
  oNode, oNewNode : TTreeNode;
begin
	oNode := tvStorages.Selected;
	if (oNode<>nil) and (oNode.ImageIndex<2) and InputQuery( 'Create new stream', 'Input stream name:', sName ) then
	try
  	OpenStream(GetNodePath(oNode)+sName,fmCreate).Destroy;
  	oNewNode := tvStorages.Items.AddChild( oNode, sName );
  	oNewNode.ImageIndex := 2;
  	oNewNode.SelectedIndex := 2;
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle, PChar('Error on creating stream'#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.miInsertStorageClick(Sender: TObject);
var
  sName : String;
  oNode, oNewNode : TTreeNode;
begin
	oNode := tvStorages.Selected;
	if (oNode<>nil) and (oNode.ImageIndex<2) and InputQuery('Create new storage(folder)', 'Input storage name', sName ) then
	try
		OpenStorage(GetNodePath(oNode)+sName,fmCreate).Close;
		oNewNode := tvStorages.Items.AddChild( oNode, sName );
		oNewNode.ImageIndex := 1;
		oNewNode.SelectedIndex := 1;
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle, PChar('Error on creating storage'#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.tvStoragesEdited(Sender: TObject; Node: TTreeNode; var S: String);
var
  sPath, sOldName : String;
  oStg : TStorage;
begin
	sOldName := Node.Text;
	try
		if Node.Level=0 then RenameFile(sOldName,S)
  	else
			begin
				sPath := GetStoragePath( Node );
				if sPath<>'' then
				begin
  				oStg := OpenStorage( sPath, fmOpenReadWrite or fmShareExclusive );
  				if oStg=nil then S := sOldName
					else
  					try
    					oStg.RenameElement( sOldName, S );
// Warning! if changed name of current stream then we must change FCurStream - this code is omitted in this sample
  					finally
    					oStg.Close;
  					end;
				end;
			end;
	except
		on E:Exception do
		begin
			S := sOldName;
			Windows.MessageBox( Application.Handle, PChar('Error on rename object'#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
		end;
	end;
end;
//-------------------------------------------------------------
procedure TfmStgView.mmStreamDataExit(Sender: TObject);
var
	oStream : TStream;
begin
	if mmStreamData.Modified and (MessageDlg('Save changes?', mtConfirmation, [mbYes,mbNo], 0)=mrYes) then
	try
		oStream := OpenStream( FCurStream, fmOpenReadWrite or fmShareExclusive );
		try
			mmStreamData.Lines.SaveToStream( oStream );
		finally
			oStream.Destroy;
		end;
	except
		on E:Exception do
			Windows.MessageBox( Application.Handle, PChar('Error on saving data in stream'#13#10+E.ClassName+': '+E.Message),
				'Error', MB_OK or MB_ICONERROR or MB_DEFBUTTON1 or MB_SETFOREGROUND or MB_TASKMODAL);
	end;
end;

end.
